library transition_profile_app_bar;

import 'package:flutter/material.dart';
import 'package:transition_profile_app_bar/avatar_curve.dart';

const double MAX_EXTENT = 310;

class TransitionProfileAppBar extends StatelessWidget {
  TransitionProfileAppBar({
    this.actionWidget1 = const SizedBox(
      height: 40,
      width: 40,
    ),
    this.actionWidget2 = const SizedBox(
      height: 40,
      width: 40,
    ),
    this.avatar,
    this.avatarActionWidget = const SizedBox(),
    this.title = const SizedBox(),
    this.backButton = const SizedBox(),
    this.additionalInfoWidgetList,
    this.appBarDecoration = const BoxDecoration(
      color: Colors.black54,
    ),
    Key key,
  }) : super(key: key);

  final Widget actionWidget1;
  final Widget actionWidget2;
  final Widget avatar;
  final Widget avatarActionWidget;
  final Widget title;
  final Widget backButton;
  final List<Widget> additionalInfoWidgetList;
  final Decoration appBarDecoration;

  @override
  Widget build(BuildContext context) {
    return SliverPersistentHeader(
      pinned: true,
      delegate: _TransitionProfileAppBarDelegate(
          actionWidget1: actionWidget1,
          actionWidget2: actionWidget2,
          avatar: avatar,
          avatarActionWidget: avatarActionWidget,
          title: title,
          backButton: backButton,
          infoList: additionalInfoWidgetList,
          appBarDecoration: appBarDecoration),
    );
  }
}

class _TransitionProfileAppBarDelegate extends SliverPersistentHeaderDelegate {
  _TransitionProfileAppBarDelegate(
      {this.actionWidget1,
      this.actionWidget2,
      this.title,
      this.backButton,
      this.avatar,
      this.avatarActionWidget,
      this.infoList,
      this.appBarDecoration});

  final Widget title;
  final Widget backButton;
  final Widget actionWidget1;
  final Widget actionWidget2;
  final Widget avatar;
  final Widget avatarActionWidget;
  final List<Widget> infoList;
  final Decoration appBarDecoration;

  double scrollAnimationValue(double shrinkOffset) {
    final double maxScrollAllowed = maxExtent - minExtent;

    return ((maxScrollAllowed - shrinkOffset) / maxScrollAllowed)
        .clamp(0, 1)
        .toDouble();
  }

  final SizeTween _avatarTween = SizeTween(
    begin: const Size(135.0, 135.0),
    end: const Size(40.0, 40.0),
  );

  @override
  Widget build(
    BuildContext context,
    double shrinkOffset,
    bool overlapsContent,
  ) {
    final double progress = shrinkOffset / MAX_EXTENT;
    final double animationVal = scrollAnimationValue(shrinkOffset);
    final Size avatarSize = _avatarTween.lerp(1 - animationVal);

    return Container(
      decoration: BoxDecoration(
        color: Colors.white,
        boxShadow: <BoxShadow>[
          BoxShadow(
            color: Colors.grey,
            blurRadius: 10 * progress,
            offset: const Offset(0.0, 0.65),
          )
        ],
      ),
      child: Stack(
        children: <Widget>[
          ClipPath(
            clipper: AvatarCurve(
              avatarSize: avatarSize,
              percentage: progress,
            ),
            child: Container(
              decoration: appBarDecoration,
            ),
          ),
          Opacity(
            opacity: 1 - progress < 0.35 ? 0 : 1,
            child: Align(
              alignment: Alignment.topLeft,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  backButton,
                  Opacity(
                    opacity: 1 - progress * 2 < 0 ? 0 : 1 - progress * 2,
                    child: title,
                  ),
                  actionWidget1,
                ],
              ),
            ),
          ),
          Positioned(
            top: 1 - progress < 0.30 ? 25 : (1 - progress) * 80,
            left: MediaQuery.of(context).size.width / 2 - avatarSize.height / 2,
            child: SizedBox.fromSize(
              size: avatarSize,
              child: Padding(
                padding: EdgeInsets.all(
                    6.0 * (1 - progress) < 1.5 ? 1.5 : 6.0 * (1 - progress)),
                child: avatar ?? Icon(Icons.person),
              ),
            ),
          ),
          Opacity(
            opacity: 1 - progress * 10 < 0 ? 0 : 1 - progress * 10,
            child: Align(
              alignment: Alignment.centerRight,
              child: Padding(
                padding: const EdgeInsets.only(top: 30.0),
                child: actionWidget2 ??
                    const SizedBox(
                      height: 10,
                    ),
              ),
            ),
          ),
          Positioned(
            top: (1 - progress) * 135,
            left: MediaQuery.of(context).size.width / 2 -
                avatarSize.width / 2 +
                (1 - progress) * 5,
            child: Opacity(
              opacity: 1 - progress * 10 < 0 ? 0 : 1 - progress * 10,
              child: SizedBox.fromSize(
                size: avatarSize,
                child: avatarActionWidget,
              ),
            ),
          ),
          infoList != null && infoList.isNotEmpty
              ? (Opacity(
                  opacity: 1 - progress * 5 < 0
                      ? progress * 2 > 1 ? 1 : progress * 2
                      : 0,
                  child: Align(
                    alignment: Alignment.bottomCenter,
                    child: Wrap(
                      direction: Axis.vertical,
                      crossAxisAlignment: WrapCrossAlignment.center,
                      children: [
                        Padding(
                          padding: EdgeInsets.only(
                              bottom: (1 - progress * 1.2) * 50.0 < 8
                                  ? 8
                                  : (1 - progress * 1.2) * 50.0),
                          child: Transform.scale(
                            scale: 0.75,
                            child: infoList[0],
                          ),
                        ),
                      ],
                    ),
                  ),
                ))
              : Container(),
          infoList != null && infoList.isNotEmpty
              ? Opacity(
                  opacity: 1 - progress * 5 < 0 ? 0 : 1 - progress * 5,
                  child: Align(
                    alignment: Alignment.bottomCenter,
                    child: Wrap(
                      direction: Axis.vertical,
                      crossAxisAlignment: WrapCrossAlignment.center,
                      children: infoList,
                    ),
                  ),
                )
              : Container(),
        ],
      ),
    );
  }

  @override
  double get maxExtent => MAX_EXTENT;

  @override
  double get minExtent => 100.0;

  @override
  bool shouldRebuild(_TransitionProfileAppBarDelegate oldDelegate) {
    return avatar != oldDelegate.avatar || infoList != oldDelegate.infoList;
  }
}
